package controller;

import javafx.scene.Group;
import javafx.scene.input.MouseEvent;
import model.*;
import view.*;

import java.util.ArrayList;

public class Log {
	// contains dynamic array of board, to be able to ctrl-Z equivalent
    ArrayList<Piece[]> logPieces;
    ArrayList<PieceView[]> logWhiteGraveyard, logBlackGraveyard;

    public Log() {
        logPieces = new ArrayList<>();
        logWhiteGraveyard = new ArrayList<>();
        logBlackGraveyard = new ArrayList<>();
    }

    public void save(Board board, BoardView boardView) {
        // we make a NEW COPY, not just getting pointer !
        Board copy = board.newInstanceCopy();
        int id = logPieces.size();
        logPieces.add(id, copy.getAllPieces());
        // for graveyards, we don't need a copy, because pieces aren't moving at all once inside, so there will be no issues
        PieceView[] white = new PieceView[16];
        PieceView[] black = new PieceView[16];
        for(int n = 0 ; n < 16 ; n++) {
            white[n] = boardView.getWhiteGraveyard()[n];
            black[n] = boardView.getBlackGraveyard()[n];
        }
        logWhiteGraveyard.add(id, white);
        logBlackGraveyard.add(id, black);

    }

    public void rollback(Board board, BoardView boardView, CustomMaterial material, Custom3dModel model, Group group3d, Game game, HUD hud) {
        int id = logPieces.size() - 1;
        if(id > 0) {
            // rollback once
            logPieces.remove(id);
            logWhiteGraveyard.remove(id);
            logBlackGraveyard.remove(id);
            // modify the game, refresh completely the board and creates pieces from log
            id--;
            board.clean(); // clean board (model)
            boardView.visualClean(group3d); // clean board (visual)
            for(int n = 0 ; n < 64 ; n++) {
                Piece piece = logPieces.get(id)[n];
                //System.out.println(piece);
                if(piece != null) {
                    board.setPiece(piece.newInstanceCopy()); // set piece (model) - new copy
                    Position p = piece.getPosition();
                    int color = piece.getColor();
                    boardView.createPiece(p, color, boardView.getMeshType(piece), material, model); // set piece (visual)
                    boardView.updatePiece(p); // place piece at right position in 3d (visual)
                    // add click-event to piece (visual)
                    PieceView pieceView = boardView.getPiece(p.getID());
                    pieceView.getObj().addEventHandler(MouseEvent.MOUSE_CLICKED, event ->
                            game.clickPiece(pieceView, board, boardView, material, model, this, hud));
                }
            }
            boardView.addPiecesOnlyToScene(group3d); // adds all the new 3d pieces to scene (visual)
            game.nextTurn(); // previous turn (same as next turn actually)
        }
        if(id == 0) { // if we are on first saving, we don't need to show rollback button anymore
            hud.rollbackHide();
        }
    }
}
