package model;

import java.util.ArrayList;


public class King extends Piece {

	public King(Position p, int color) {
		super(p, color);
	}

	public ArrayList<Position> getAvailableMoves(Board b) {

		// creates a dynamic list of available moves
		ArrayList<Position> moves = new ArrayList<>();
		Position arrival;

		// list of the 8 king movements
		int[] all_x = {0, 1, 1, 1, 0, -1, -1, -1};
		int[] all_y = {1, 1, 0, -1, -1, -1, 0, 1};

		for(int n = 0 ; n < all_x.length ; n++) { // for each direction
			int x = all_x[n];
			int y = all_y[n];
			arrival = new Position(p.getX() + x, p.getY() + y);
			if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
				moves.add(arrival);
			}
		}

		// Castling
		if(!hasBeenMoved) {
			Position rookWest = new Position(1, p.getY());
			if(	b.isRook(rookWest) && !b.getPiece(rookWest).hasBeenMoved()
					&& b.isFree(new Position(2, p.getY()))
					&& b.isFree(new Position(3, p.getY()))
					&& b.isFree(new Position(4, p.getY())) ) {
				moves.add(rookWest);
			}
			Position rookEast = new Position(8, p.getY());
			if(	b.isRook(rookEast) && !b.getPiece(rookEast).hasBeenMoved()
					&& b.isFree(new Position(7, p.getY()))
					&& b.isFree(new Position(6, p.getY())) ) {
				moves.add(rookEast);
			}
		}

		// Full wrote version : --------------------------------------------
		/*
		// up
		arrival = new Position(p.getX(), p.getY() + 1);
		if(b.isEnemy(arrival,p) || b.isFree(arrival)) { moves.add(arrival); }

		//down
		arrival = new Position(p.getX(), p.getY() - 1);
		if(b.isEnemy(arrival,p) || b.isFree(arrival)) { moves.add(arrival); }

		//left
		arrival = new Position(p.getX()+1, p.getY());
		if(b.isEnemy(arrival,p) || b.isFree(arrival)) { moves.add(arrival); }

		//right
		arrival = new Position(p.getX()-1, p.getY());
		if(b.isEnemy(arrival,p) || b.isFree(arrival)) { moves.add(arrival); }

		//up left
		arrival = new Position(p.getX()-1, p.getY() + 1);
		if(b.isEnemy(arrival,p) || b.isFree(arrival)) { moves.add(arrival); }

		//up right
		arrival = new Position(p.getX()+1, p.getY() + 1);
		if(b.isEnemy(arrival,p) || b.isFree(arrival)) { moves.add(arrival); }

		//down left
		arrival = new Position(p.getX()-1, p.getY() -1);
		if(b.isEnemy(arrival,p) || b.isFree(arrival)) { moves.add(arrival); }

		//down right
		arrival = new Position(p.getX()+1, p.getY() - 1);
		if(b.isEnemy(arrival,p) || b.isFree(arrival)) { moves.add(arrival); }

		arrival = new Position(p.getX(), p.getY() + 1);
		if(b.isEnemy(arrival,p) || b.isFree(arrival)) { moves.add(arrival); }
		*/
		// ---------------------------------------------------------------

		return moves;
	}

	public Piece newInstanceCopy() {
		return new King(p, color);
	}
}

