package view;

import javafx.scene.Group;
import javafx.scene.shape.Box;

public class Room {
    private final Box[] walls, ground;
    private final Box ceiling;

    private final static int WALL_N = 0, WALL_E = 1, WALL_S = 2, WALL_W = 3, GROUND = 4, CEILING = 5;
    private final static int GROUND_X = 8, GROUND_Y = 8;

    public Room(CustomMaterial material) {

        int size = BoardView.BOARD_SIZE*60;
        double height = 0.5;

        int[] pos_x = {0, 1, 0, -1, 0, 0};
        int[] pos_y = {0, 0, 0, 0, 1, -1};
        int[] pos_z = {1, 0, -1, 0, 0, 0};
        int[] size_x = {1, 0, 1, 0, 1, 1};
        int[] size_y = {1, 1, 1, 1, 0, 0};
        int[] size_z = {0, 1, 0, 1, 1, 1};

        // Ground
        ground = new Box[8*8];
        for(int j = 0 ; j < GROUND_Y ; j++) {
            for (int i = 0; i < GROUND_X; i++) {
                int n = j * 8 + i;
                ground[n] = new Box(1 + size_x[GROUND] * size / GROUND_X, 1 + size_y[GROUND] * size * height, 1 + size_z[GROUND] * size / GROUND_Y);
                ground[n].translateXProperty().set((double) /*pos_x[GROUND] * (size / 2) +*/ (i * size / GROUND_X) - (size / 2) + (size / GROUND_X / 2));
                ground[n].translateYProperty().set((double) /*(pos_y[GROUND] * (size / 2) - (size / 2)) * height +*/ BoardView.BOARD_SIZE * 9); // centered up, then down to table legs
                ground[n].translateZProperty().set((double) /*pos_z[GROUND] * (size / 2) +*/ (j * size / GROUND_Y) - (size / 2) + (size / GROUND_Y / 2));
                ground[n].setMaterial(material.get(CustomMaterial.CARPET));
            }
        }

        // Ceiling
        ceiling = new Box(1 + size_x[CEILING]*size, 1 + size_y[CEILING]*size*height, 1 + size_z[CEILING]*size);
        ceiling.translateXProperty().set((double) pos_x[CEILING] * (size/2));
        ceiling.translateYProperty().set((double) (pos_y[CEILING] * (size/2) - (size/2)) * height + BoardView.BOARD_SIZE*9); // centered up, then down to table legs
        ceiling.translateZProperty().set((double) pos_z[CEILING] * (size/2));
        ceiling.setMaterial(material.get(CustomMaterial.WHITE));

        // Walls
        walls = new Box[4];
        for(int n = 0 ; n < 4 ; n++) {
            walls[n] = new Box(1 + size_x[n]*size, 1 + size_y[n]*size*height, 1 + size_z[n]*size);
            walls[n].translateXProperty().set((double) pos_x[n] * (size/2));
            walls[n].translateYProperty().set((double) (pos_y[n] * (size/2) - (size/2)) * height + BoardView.BOARD_SIZE*9); // centered up, then down to table legs
            walls[n].translateZProperty().set((double) pos_z[n] * (size/2));
            walls[n].setMaterial(material.get(CustomMaterial.STUDIO_WALL));
        }
    }

    public void addAllToScene(Group group) {
        // Ground
        for(int n = 0 ; n < 8*8 ; n++) {
            group.getChildren().add(ground[n]);
        }
        // Ceiling
        group.getChildren().add(ceiling);
        // Walls
        for(int n = 0 ; n < 4 ; n++) {
            group.getChildren().add(walls[n]);
        }
    }
}
